
# q-expansion of newform 320.3.b.c, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 320
weight = 3
poly_data = [1, -1, 1, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 4, 4], 1], [[1, 0, 2, -2], 1], [[-2, 4, -2, 2], 1]]

hecke_ring_character_values = [[191, [-1, 0, 0, 0]], [261, [1, 0, 0, 0]], [257, [1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 1], [0, 0, 1, 0], [0, -1, 0, 1], [0, -2, 0, -2], [4, 0, 2, 0], [-6, 0, 8, 0], [0, 2, 0, 0], [0, -1, 0, 3], [2, 0, 4, 0], [0, 2, 0, -10], [-4, 0, 10, 0], [-28, 0, 6, 0], [0, -2, 0, -3], [0, 3, 0, 13], [44, 0, 10, 0], [0, -6, 0, 12], [-32, 0, -26, 0], [0, -10, 0, 11], [0, -10, 0, -2], [66, 0, -32, 0], [0, -8, 0, 20], [0, 6, 0, 13], [-22, 0, -40, 0], [-66, 0, -12, 0], [-82, 0, 8, 0], [0, 11, 0, 23], [0, 12, 0, -17], [32, 0, -74, 0], [126, 0, 20, 0], [0, -19, 0, -23], [0, -14, 0, 42], [-26, 0, 12, 0], [0, 30, 0, 4], [-48, 0, 38, 0], [0, 18, 0, 62], [-84, 0, -54, 0], [0, -28, 0, 45], [0, 9, 0, -15], [204, 0, 10, 0], [0, -6, 0, -76], [-122, 0, 16, 0], [0, -22, 0, 58], [146, 0, -16, 0], [16, 0, 70, 0], [0, -16, 0, -64], [0, 14, 0, 42], [0, 35, 0, -13], [0, 8, 0, -1], [-118, 0, -20, 0], [6, 0, 140, 0], [0, 20, 0, 36], [-68, 0, 30, 0], [0, 26, 0, -26], [-266, 0, -4, 0], [0, 29, 0, 79], [192, 0, 70, 0], [0, -6, 0, -86], [256, 0, -90, 0], [-28, 0, -10, 0], [0, 24, 0, -15], [-76, 0, -46, 0], [0, 34, 0, 11], [0, 34, 0, 10], [-174, 0, -176, 0], [-264, 0, 46, 0], [0, -22, 0, -70], [-126, 0, -64, 0], [0, 38, 0, 91], [242, 0, -108, 0], [-14, 0, -144, 0], [0, 8, 0, 16], [0, 17, 0, 157], [-256, 0, -154, 0], [0, -30, 0, -32], [0, -57, 0, 43], [-48, 0, -154, 0], [-124, 0, -94, 0], [282, 0, -72, 0], [-272, 0, -150, 0], [0, 70, 0, -28], [328, 0, -114, 0], [0, -22, 0, -126], [-234, 0, 60, 0], [0, 64, 0, -48], [0, 44, 0, -19], [528, 0, -102, 0], [554, 0, 120, 0], [278, 0, 80, 0], [0, 1, 0, 167], [0, -116, 0, -13], [0, -16, 0, -172], [0, 79, 0, -67], [0, 26, 0, 38], [0, -114, 0, 24], [0, 107, 0, -73], [602, 0, 140, 0], [202, 0, 360, 0], [0, 52, 0, 161], [438, 0, -80, 0], [0, 20, 0, -1], [-704, 0, 86, 0], [0, -92, 0, 89], [-112, 0, -54, 0], [0, -10, 0, 122], [-186, 0, 236, 0], [0, -34, 0, -153], [-94, 0, 96, 0], [0, -108, 0, -12], [172, 0, -82, 0], [0, -69, 0, -251], [-656, 0, 70, 0], [54, 0, 220, 0], [0, 2, 0, -200], [0, -26, 0, 198], [212, 0, -282, 0], [0, 26, 0, -271], [0, 19, 0, 117], [464, 0, -42, 0], [0, 14, 0, 92], [48, 0, 342, 0], [-114, 0, 260, 0], [136, 0, -306, 0], [0, -22, 0, 61], [0, -2, 0, 86], [568, 0, 62, 0], [-518, 0, 316, 0], [0, 44, 0, -76], [0, -77, 0, -55], [-236, 0, -446, 0], [0, 10, 0, -36], [0, -67, 0, -213], [0, -58, 0, -282], [576, 0, 134, 0], [122, 0, -344, 0], [-102, 0, 328, 0], [104, 0, 142, 0], [0, -58, 0, -169], [-744, 0, -210, 0], [218, 0, -200, 0], [0, -22, 0, -122], [-232, 0, -402, 0], [0, 67, 0, 115], [0, -110, 0, 267], [352, 0, 70, 0], [0, -104, 0, 220], [264, 0, 590, 0], [-426, 0, -260, 0], [0, -6, 0, 52], [0, -79, 0, 147], [-284, 0, -142, 0], [-348, 0, 262, 0], [0, 32, 0, -103], [0, -9, 0, -199], [0, -52, 0, 27], [0, -54, 0, -442], [0, 44, 0, 204], [968, 0, -254, 0], [294, 0, -20, 0], [-1082, 0, 160, 0], [0, 70, 0, 259], [866, 0, 192, 0], [0, -91, 0, 125], [0, 198, 0, -246], [-1226, 0, -68, 0], [0, 61, 0, -65], [0, 122, 0, 14], [316, 0, 346, 0]]
